<?php
header('Content-Type: application/json');
require 'vendor/autoload.php';
date_default_timezone_set('Asia/Dhaka');
include('database.php');

// Function to extract images
function extractImagesFromPdf($pdfFilePath, $name, $nid) {
    $blackfiretools = new \Smalot\PdfParser\Parser();
    $pdf = $blackfiretools->parseFile($pdfFilePath);

    $images = $pdf->getObjectsByType('XObject', 'Image');
    $outputDir = __DIR__ . '/images/';

    if (!file_exists($outputDir)) {
        mkdir($outputDir, 0777, true);
    }

    $imageNames = [];
    $counter = 0;

    foreach ($images as $image) {
        $imageData = $image->getContent();
        $tempImagePath = $outputDir . 'temp_image_' . uniqid() . '.tmp';
        file_put_contents($tempImagePath, $imageData);
        $imageInfo = getimagesize($tempImagePath);

        if ($imageInfo === false) {
            unlink($tempImagePath);
            if ($counter === 1) {
                $imageNames[] = signature($name, $nid);
            }
            continue;
        }

        $imageExtension = image_type_to_extension($imageInfo[2], false);
        $validExtensions = ['jpeg', 'png', 'gif'];

        if (!in_array($imageExtension, $validExtensions)) {
            unlink($tempImagePath);
            continue;
        }

        if ($counter === 0) {
            $imageFileName = $nid . '-user.' . $imageExtension;
        } elseif ($counter === 1) {
            $imageFileName = $nid . '-sign.' . $imageExtension;
        } else {
            $imageFileName = $nid . '-image' . ($counter + 1) . '.' . $imageExtension;
        }

        $imageFilePath = $outputDir . $imageFileName;
        rename($tempImagePath, $imageFilePath);

        $imageNames[] = $imageFileName;
        $counter++;
    }

    // If the second image is missing, generate a signature
    if (!isset($imageNames[1])) {
        $imageNames[1] = signature($name, $nid);
    }

    return $imageNames;
}

function extractPdfData($filePath) {
    try {
        $blackfiretools = new Smalot\PdfParser\Parser();
        $pdf = $blackfiretools->parseFile($filePath);
        $text = $pdf->getText();

        $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https://' : 'http://';
        $domain = $_SERVER['SERVER_NAME'];

        $imageNames = extractImagesFromPdf($filePath, extractField($text, "Name(Bangla)", "Name(English)"), extractField($text, "National ID", "Pin"));

        $signImg = $protocol . $domain . "/sign/images/" . $imageNames[1];


        $data = [
            "nameBangla" => extractField($text, "Name(Bangla)", "Name(English)"),
            "nameEnglish" => strtoupper(extractField($text, "Name(English)", "Date of Birth")),
            "nationalId" => extractField($text, "National ID", "Pin"),
            "pin" => extractField($text, "Pin", "Status"),
            "dateOfBirth" => date('d M Y', strtotime(extractField($text, "Date of Birth", "\n"))),
            "dateOfToday" => convertToBangla(date('d-m-Y', time())),
            "fatherName" => !empty(extractField($text, "Father Name", "\n"))
            ? extractField($text, "Father Name", "\n")
            : extractField($text, "Father Name", "Mother Name"),
            "motherName" => !empty(extractField($text, "Mother Name", "\n"))
            ? extractField($text, "Mother Name", "\n")
            : extractField($text, "Mother Name", "Spouse Name"),
            "gender" => !empty(extractField($text, "Gender", "\n"))
            ? extractField($text, "Gender", "\n")
            : extractField($text, "Gender", "Marital"),
            "religion" => !empty(extractField($text, "Religion", "\n"))
            ? extractField($text, "Religion", "\n")
            : extractField($text, "Religion", "Religion"),
            "birthPlace" => !empty(extractField($text, "Birth Place", "\n"))
            ? extractField($text, "Birth Place", "\n")
            : extractField($text, "Birth Place", "Birth Other"),
            "bloodGroup" => !empty(extractField($text, "Blood Group", "\n"))
            ? extractField($text, "Blood Group", "\n")
            : extractField($text, "Blood Group", "TIN"),
            "userIMG" => $protocol . $domain . "/sign/images/" . $imageNames[0],
            "signIMG" => $signImg,
            "address" => combineAddress($text)
        ];

        return [
            "code" => 200,
            "success" => true,
            "message" => "Data fatched successfully",
            "data" => $data,

        ];
    } catch (Exception $e) {
        return [
            "code" => 500,
            "success" => false,
            "message" => "Error processing PDF" . $e->getMessage()
        ];
    }
}

function extractField($text, $start, $end) {
    $pattern = '/' . preg_quote($start, '/') . '(.*?)' . preg_quote($end, '/') . '/s';
    if (preg_match($pattern, $text, $matches)) {
        return trim($matches[1]);
    }
    return null;
}

function convertToBangla($number) {
    $englishDigits = [
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9'];
    $banglaDigits = [
        '০',
        '১',
        '২',
        '৩',
        '৪',
        '৫',
        '৬',
        '৭',
        '৮',
        '৯'];
    return str_replace($englishDigits, $banglaDigits, $number);
}

function combineAddress($text) {
    $home = extractField($text, "Home/Holding", "Post");
    $village = extractField($text, "Village/Road", "\n");
    $village = str_replace("Village/Road", "", $village);
    $postOffice = extractField($text, "Post Office", "Postal Code");
    $postalCode = extractField($text, "Postal Code", "Region");
    $upozila = extractField($text, "Upozila", "Union/Ward");
    $district = extractField($text, "District", "RMO");

    if (empty($village)) {
        $village = extractField($text, "Additional\nVillage/Road", "Home/Holding");
    }

    $resp = "বাসা/হোল্ডিং: $home, গ্রাম/রাস্তা: " . str_replace(" ", "", $village) . ", ডাকঘর: $postOffice -" . convertToBangla($postalCode) . ", $upozila, $district";
    $resp = str_replace(
        ["\t", "\n", 'Additional', 'No'],
        ['', ' ', '', ''],
        $resp
    );
    return trim(str_replace("  ", " ", $resp));
}

function getMiddlePart($name) {
    $nameParts = explode(' ', trim($name));
    $count = count($nameParts);

    if ($count < 3) {
        return $nameParts[0];
    }

    $middleParts = array_slice($nameParts, 1, $count - 2);

    return implode(' ', $middleParts);
}

function signature($name, $nid) {
    $name = getMiddlePart($name);
    $url = "https://yournodejslink.com/signature?name=" . urlencode($name) . "&nid=" . urlencode($nid);
    $resp = json_decode(file_get_contents($url), true);
    $imageUrl = $resp['imageUrl'];
    $imageName = basename($imageUrl);
    $savePath = __DIR__ . '/images/' . $imageName;
    $imageContent = file_get_contents($imageUrl);
    if ($imageContent !== false) {
        file_put_contents($savePath, $imageContent);
    } else {
        throw new Exception("Failed to download the image from $imageUrl");
    }
    return $imageName;
}

function printPDFdata() {
    $uploadDir = __DIR__ . '/uploads/';
    if (!file_exists($uploadDir)) mkdir($uploadDir, 0777, true);

    if (!isset($_FILES['pdf']) || $_FILES['pdf']['error'] !== UPLOAD_ERR_OK) {
        http_response_code(400);
        echo json_encode(['error' => 'No file uploaded or upload error occurred.']);
        exit;
    }

    $fileTmpPath = $_FILES['pdf']['tmp_name'];
    $fileName = basename($_FILES['pdf']['name']);
    $fileInfo = pathinfo($fileName);

    if (strtolower($fileInfo['extension']) !== 'pdf') {
        http_response_code(400);
        echo json_encode(['error' => 'Invalid file type. Only PDF files are allowed.']);
        exit;
    }

    $uFileName = md5(uniqid($fileInfo['filename'], true)) . '.pdf';
    $uploadedFilePath = $uploadDir . $uFileName;

    if (!move_uploaded_file($fileTmpPath, $uploadedFilePath)) {
        http_response_code(500);
        echo json_encode(['error' => 'Failed to move uploaded file.']);
        exit;
    }

    try {
        $response = extractPdfData($uploadedFilePath);
        http_response_code(200);
        echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
        return true;
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['error' => 'Error processing the PDF' . $e->getMessage()]);
    } finally {
        unlink($uploadedFilePath);
    }
}


$key = isset($_REQUEST['key']) ? $_REQUEST['key'] : null;

if (empty($key)) {
    // Fixed missing closing parenthesis
    http_response_code(400);
    echo json_encode([
        "success" => false,
        "code" => 400,
        "message" => "Missing required parameter key"
    ], JSON_PRETTY_PRINT);
    exit;
}


$sql = "SELECT * FROM users WHERE apiKey = ?";
if ($stmt = $conn->prepare($sql)) {
    $stmt->bind_param("s", $key);
    $stmt->execute();
    $result = $stmt->get_result();

    if ($result->num_rows === 1) {
        $user = $result->fetch_assoc();
        $username = $user['username'];
        $expDate = $user['expiryDate'];
        $status = strtolower($user['status']);
        $balance = $user['balance'];

        $currentDateTime = date('Y-m-d H:i:s');

        if ($expDate < $currentDateTime) {
            http_response_code(403);
            echo json_encode([
                'error' => 'API key expired',
                'expDate' => $expDate,
                'details' => 'Your API key has expired. Please contact admin to renew.'
            ], JSON_PRETTY_PRINT);
            $stmt->close();
            $conn->close();
            exit;
        }

        if ($status !== 'on') {
            http_response_code(403);
            echo json_encode([
                'error' => 'This key is deactivated.',
                'keyStatus' => $status,
                'details' => 'Contact Admin To Activate Key'
            ], JSON_PRETTY_PRINT);
            $stmt->close();
            $conn->close();
            exit;
        }

        if ($balance <= 0 && $balance !== 99999999) {
            http_response_code(403);
            echo json_encode([
                'error' => 'Insufficient balance.',
                'balance' => $balance,
                'details' => 'Contact Admin To Buy Balance'
            ], JSON_PRETTY_PRINT);
            $stmt->close();
            $conn->close();
            exit;
        }

        if ($balance > 0) {
            $isSuccess = printPDFdata();

            if ($isSuccess) {
                $updateSql = "UPDATE users SET balance = balance - 1, used = used + 1 WHERE apiKey = ?";
                if ($updateStmt = $conn->prepare($updateSql)) {
                    $updateStmt->bind_param("s", $key);
                    $updateStmt->execute();
                    $updateStmt->close();
                }
            } else {
                http_response_code(500);
                echo json_encode([
                    'error' => 'Failed to process the PDF.',
                    'message' => 'An internal error occurred. Please try again later.'
                ], JSON_PRETTY_PRINT);
                $stmt->close();
                $conn->close();
                exit;
            }
        } else {
            http_response_code(400);
            echo json_encode([
                'error' => 'Insufficient balance.',
                'message' => 'Contact Admin To Buy Balance',
                'API_OWNER' => '@Owner_Of_BFT'
            ], JSON_PRETTY_PRINT);
        }
    } else {
        http_response_code(401);
        echo json_encode([
            'error' => 'Invalid API key.',
            'message' => 'Unauthorized access',
            'API_OWNER' => '@Owner_Of_BFT'
        ], JSON_PRETTY_PRINT);
    }

    $stmt->close();
} else {
    // Database error response
    http_response_code(500);
    echo json_encode([
        'error' => 'Database error!',
        'message' => 'An unexpected error occurred. Please report this to the admin.'
    ], JSON_PRETTY_PRINT);
}

$conn->close();
?>